@@ -14,23 +14,25 @@ typedef enum
 {
   DEBUG_PRESENCE       = 1 << 0,
   DEBUG_GROUPS         = 1 << 1,
-  DEBUG_CONTACTS       = 1 << 2,
-  DEBUG_DISCO          = 1 << 3,
-  DEBUG_PROPERTIES     = 1 << 4,
-  DEBUG_ROOMLIST       = 1 << 5,
-  DEBUG_MEDIA          = 1 << 6,
-  DEBUG_MUC            = 1 << 7,
-  DEBUG_MUC_CONNECTION = 1 << 8,
-  DEBUG_CONNECTION     = 1 << 9,
-  DEBUG_IM             = 1 << 10,
-  DEBUG_SI_BYTESTREAM_MGR = 1 << 11,
-  DEBUG_NET            = 1 << 12,
-  DEBUG_SELF           = 1 << 13,
-  DEBUG_TUBES          = 1 << 14,
-  DEBUG_XCM            = 1 << 15,
-  DEBUG_DISCOVERY      = 1 << 16,
-  DEBUG_OLPC_ACTIVITY  = 1 << 17,
-  DEBUG_FT             = 1 << 18,
+  DEBUG_CAPS           = 1 << 2,
+  DEBUG_CONTACTS       = 1 << 3,
+  DEBUG_DISCO          = 1 << 4,
+  DEBUG_PROPERTIES     = 1 << 5,
+  DEBUG_ROOMLIST       = 1 << 6,
+  DEBUG_MEDIA          = 1 << 7,
+  DEBUG_MUC            = 1 << 8,
+  DEBUG_MUC_CONNECTION = 1 << 9,
+  DEBUG_CONNECTION     = 1 << 10,
+  DEBUG_IM             = 1 << 11,
+  DEBUG_SI_BYTESTREAM_MGR     = 1 << 12,
+  DEBUG_DIRECT_BYTESTREAM_MGR = 1 << 13,
+  DEBUG_NET            = 1 << 14,
+  DEBUG_SELF           = 1 << 15,
+  DEBUG_TUBES          = 1 << 16,
+  DEBUG_XCM            = 1 << 17,
+  DEBUG_DISCOVERY      = 1 << 18,
+  DEBUG_OLPC_ACTIVITY  = 1 << 19,
+  DEBUG_FT             = 1 << 20,
 } DebugFlags;
 
 void debug_set_flags_from_env (void);
