@@ -29,6 +29,7 @@
 #include <gibber/gibber-namespaces.h>
 #include <gibber/gibber-xmpp-error.h>
 
+#include "salut-caps-channel-manager.h"
 #include "salut-muc-channel.h"
 #include "salut-contact-manager.h"
 #include "salut-tubes-channel.h"
@@ -36,9 +37,11 @@
 #include "salut-roomlist-manager.h"
 #include "salut-xmpp-connection-manager.h"
 #include "salut-discovery-client.h"
+#include "tube-stream.h"
 
 #include <telepathy-glib/channel-manager.h>
 #include <telepathy-glib/dbus.h>
+#include <telepathy-glib/gtypes.h>
 #include <telepathy-glib/interfaces.h>
 #include <telepathy-glib/util.h>
 
@@ -62,7 +65,9 @@ static void salut_muc_manager_iface_init (gpointer g_iface,
 G_DEFINE_TYPE_WITH_CODE(SalutMucManager, salut_muc_manager,
     G_TYPE_OBJECT,
     G_IMPLEMENT_INTERFACE (TP_TYPE_CHANNEL_MANAGER,
-      salut_muc_manager_iface_init));
+      salut_muc_manager_iface_init);
+    G_IMPLEMENT_INTERFACE (SALUT_TYPE_CAPS_CHANNEL_MANAGER,
+      NULL));
 
 /* properties */
 enum {
@@ -363,14 +368,22 @@ salut_muc_manager_foreach_channel_class (TpChannelManager *manager,
   g_hash_table_insert (table, TP_IFACE_CHANNEL ".TargetHandleType",
       handle_type_value);
 
+  /* org.freedesktop.Telepathy.Channel.Type.Text */
   g_value_set_static_string (channel_type_value, TP_IFACE_CHANNEL_TYPE_TEXT);
   func (manager, table, muc_channel_allowed_properties,
       user_data);
 
+  /* org.freedesktop.Telepathy.Channel.Type.Tubes */
   g_value_set_static_string (channel_type_value, TP_IFACE_CHANNEL_TYPE_TUBES);
   func (manager, table, muc_tubes_channel_allowed_properties,
       user_data);
 
+  /* org.freedesktop.Telepathy.Channel.Type.StreamTube */
+  g_value_set_static_string (channel_type_value,
+      SALUT_IFACE_CHANNEL_TYPE_STREAM_TUBE);
+  func (manager, table, salut_tube_stream_channel_get_allowed_properties (),
+      user_data);
+
   g_hash_table_destroy (table);
 }
 
@@ -495,7 +508,8 @@ static SalutTubesChannel *
 new_tubes_channel (SalutMucManager *self,
                    TpHandle room,
                    SalutMucChannel *muc,
-                   TpHandle initiator)
+                   TpHandle initiator,
+                   gboolean requested)
 {
   SalutMucManagerPrivate *priv = SALUT_MUC_MANAGER_GET_PRIVATE (self);
   TpBaseConnection *conn = (TpBaseConnection *) priv->connection;
@@ -517,6 +531,7 @@ new_tubes_channel (SalutMucManager *self,
       "handle-type", TP_HANDLE_TYPE_ROOM,
       "muc", muc,
       "initiator-handle", initiator,
+      "requested", requested,
       NULL);
 
   g_signal_connect (chan, "closed", (GCallback) tubes_channel_closed_cb, self);
@@ -532,6 +547,7 @@ static SalutMucChannel *
 salut_muc_manager_request_new_muc_channel (SalutMucManager *mgr,
                                            TpHandle handle,
                                            gpointer request_token,
+                                           gboolean announce,
                                            GError **error)
 {
   SalutMucManagerPrivate *priv = SALUT_MUC_MANAGER_GET_PRIVATE (mgr);
@@ -616,8 +632,11 @@ salut_muc_manager_request_new_muc_channel (SalutMucManager *mgr,
   if (request_token != NULL)
     tokens = g_slist_prepend (tokens, request_token);
 
-  tp_channel_manager_emit_new_channel (mgr, TP_EXPORTABLE_CHANNEL (text_chan),
-      tokens);
+  if (announce)
+    {
+      tp_channel_manager_emit_new_channel (mgr,
+          TP_EXPORTABLE_CHANNEL (text_chan), tokens);
+    }
 
   g_slist_free (tokens);
 
@@ -629,12 +648,15 @@ create_tubes_channel (SalutMucManager *self,
                       TpHandle handle,
                       TpHandle initiator,
                       gpointer request_token,
+                      gboolean announce,
+                      gboolean *text_created_out,
+                      gboolean requested,
                       GError **error)
 {
   SalutMucManagerPrivate *priv = SALUT_MUC_MANAGER_GET_PRIVATE (self);
   SalutMucChannel *text_chan;
   SalutTubesChannel *tubes_chan;
-  GSList *tokens = NULL;
+  gboolean text_created = FALSE;
 
   text_chan = g_hash_table_lookup (priv->text_channels,
       GUINT_TO_POINTER (handle));
@@ -643,22 +665,44 @@ create_tubes_channel (SalutMucManager *self,
     {
       DEBUG ("have to create the text channel before the tubes one");
       text_chan = salut_muc_manager_request_new_muc_channel (self,
-          handle, NULL, error);
+          handle, NULL, FALSE, error);
 
       if (text_chan == NULL)
         return NULL;
+
+      text_created = TRUE;
     }
 
-  tubes_chan = new_tubes_channel (self, handle, text_chan, initiator);
+  tubes_chan = new_tubes_channel (self, handle, text_chan, initiator,
+      requested);
   g_assert (tubes_chan != NULL);
 
-  if (request_token != NULL)
-    tokens = g_slist_prepend (tokens, request_token);
+  if (announce)
+    {
+      GHashTable *channels;
+      GSList *tokens = NULL;
 
-  tp_channel_manager_emit_new_channel (self, TP_EXPORTABLE_CHANNEL (tubes_chan),
-      tokens);
+      if (request_token != NULL)
+        tokens = g_slist_prepend (tokens, request_token);
 
-  g_slist_free (tokens);
+      /* announce channels */
+      channels = g_hash_table_new_full (g_direct_hash, g_direct_equal,
+          NULL, NULL);
+
+      if (text_created)
+        {
+          g_hash_table_insert (channels, text_chan, NULL);
+        }
+
+      g_hash_table_insert (channels, tubes_chan, tokens);
+      tp_channel_manager_emit_new_channels (self, channels);
+
+      g_hash_table_destroy (channels);
+      g_slist_free (tokens);
+    }
+
+  if (text_created_out != NULL)
+    *text_created_out = text_created;
 
   return tubes_chan;
 }
@@ -671,8 +715,6 @@ salut_muc_manager_request (SalutMucManager *self,
 {
   SalutMucManagerPrivate *priv = SALUT_MUC_MANAGER_GET_PRIVATE (self);
   TpBaseConnection *base_conn = (TpBaseConnection *) priv->connection;
-  TpHandleRepoIface *room_repo = tp_base_connection_get_handles (base_conn,
-      TP_HANDLE_TYPE_ROOM);
   GError *error = NULL;
   TpHandle handle;
   const gchar *channel_type;
@@ -683,15 +725,19 @@ salut_muc_manager_request (SalutMucManager *self,
       TP_IFACE_CHANNEL ".TargetHandleType", NULL) != TP_HANDLE_TYPE_ROOM)
     return FALSE;
 
-  handle = tp_asv_get_uint32 (request_properties,
-      TP_IFACE_CHANNEL ".TargetHandle", NULL);
-
-  if (!tp_handle_is_valid (room_repo, handle, &error))
-    goto error;
-
   channel_type = tp_asv_get_string (request_properties,
       TP_IFACE_CHANNEL ".ChannelType");
 
+  if (tp_strdiff (channel_type, TP_IFACE_CHANNEL_TYPE_TEXT) &&
+      tp_strdiff (channel_type, TP_IFACE_CHANNEL_TYPE_TUBES) &&
+      tp_strdiff (channel_type, SALUT_IFACE_CHANNEL_TYPE_STREAM_TUBE))
+    return FALSE;
+
+  /* validity already checked by TpBaseConnection */
+  handle = tp_asv_get_uint32 (request_properties,
+      TP_IFACE_CHANNEL ".TargetHandle", NULL);
+  g_assert (handle != 0);
+
   if (!tp_strdiff (channel_type, TP_IFACE_CHANNEL_TYPE_TEXT))
     {
       if (tp_channel_manager_asv_has_unknown_properties (request_properties,
@@ -719,7 +765,7 @@ salut_muc_manager_request (SalutMucManager *self,
       else
         {
           text_chan = salut_muc_manager_request_new_muc_channel (self,
-              handle, request_token, NULL);
+              handle, request_token, TRUE, NULL);
         }
 
       return TRUE;
@@ -752,13 +798,80 @@ salut_muc_manager_request (SalutMucManager *self,
       else
         {
           tubes_chan = create_tubes_channel (self, handle,
-              base_conn->self_handle, request_token, &error);
+              base_conn->self_handle, request_token, TRUE, NULL, TRUE, &error);
           if (tubes_chan == NULL)
             goto error;
         }
 
       return TRUE;
     }
+  else if (!tp_strdiff (channel_type, SALUT_IFACE_CHANNEL_TYPE_STREAM_TUBE))
+    {
+      const gchar *service;
+      SalutTubeIface *new_channel;
+      GHashTable *channels;
+      GSList *request_tokens;
+      gboolean announce_text = FALSE, announce_tubes = FALSE;
+
+      if (tp_channel_manager_asv_has_unknown_properties (request_properties,
+              muc_tubes_channel_fixed_properties,
+              salut_tube_stream_channel_get_allowed_properties (),
+              &error))
+        goto error;
+
+      /* "Service" is a mandatory, not-fixed property */
+      service = tp_asv_get_string (request_properties,
+                SALUT_IFACE_CHANNEL_TYPE_STREAM_TUBE ".Service");
+      if (service == NULL)
+        {
+          g_set_error (&error, TP_ERRORS, TP_ERROR_NOT_IMPLEMENTED,
+              "Request does not contain the mandatory property '%s'",
+              SALUT_IFACE_CHANNEL_TYPE_STREAM_TUBE ".Service");
+          goto error;
+        }
+
+      tubes_chan = g_hash_table_lookup (priv->tubes_channels,
+          GUINT_TO_POINTER (handle));
+      if (tubes_chan == NULL)
+        {
+          tubes_chan = create_tubes_channel (self, handle,
+              base_conn->self_handle, NULL, FALSE, &announce_text,
+              FALSE, &error);
+          if (tubes_chan == NULL)
+            goto error;
+          announce_tubes = TRUE;
+        }
+
+      g_assert (tubes_chan != NULL);
+      new_channel = salut_tubes_channel_tube_request (tubes_chan, channel_type,
+          service);
+      g_assert (new_channel != NULL);
+
+      /* announce channels */
+      channels = g_hash_table_new_full (g_direct_hash, g_direct_equal,
+          NULL, NULL);
+
+      if (announce_text)
+        {
+          text_chan = g_hash_table_lookup (priv->text_channels,
+              GINT_TO_POINTER (handle));
+          g_assert (text_chan != NULL);
+          g_hash_table_insert (channels, text_chan, NULL);
+        }
+
+      if (announce_tubes)
+        {
+          g_hash_table_insert (channels, tubes_chan, NULL);
+        }
+
+      request_tokens = g_slist_prepend (NULL, request_token);
+      g_hash_table_insert (channels, new_channel, request_tokens);
+      tp_channel_manager_emit_new_channels (self, channels);
+
+      g_hash_table_destroy (channels);
+      g_slist_free (request_tokens);
+      return TRUE;
+    }
   else
     {
       return FALSE;
@@ -1004,10 +1117,12 @@ salut_muc_manager_handle_si_stream_request (SalutMucManager *self,
   salut_tubes_channel_bytestream_offered (chan, bytestream, msg);
 }
 
+/* Caller is reponsible of announcing the channel if created */
 SalutTubesChannel *
 salut_muc_manager_ensure_tubes_channel (SalutMucManager *self,
                                         TpHandle handle,
-                                        TpHandle actor)
+                                        TpHandle actor,
+                                        gboolean *created)
 {
   SalutMucManagerPrivate *priv = SALUT_MUC_MANAGER_GET_PRIVATE (self);
   SalutTubesChannel *tubes_chan;
@@ -1017,12 +1132,16 @@ salut_muc_manager_ensure_tubes_channel (SalutMucManager *self,
   if (tubes_chan != NULL)
     {
       g_object_ref (tubes_chan);
+      *created = FALSE;
       return tubes_chan;
     }
 
-  tubes_chan = create_tubes_channel (self, handle, actor, NULL, NULL);
+
+  tubes_chan = create_tubes_channel (self, handle, actor, NULL, FALSE, NULL,
+      FALSE, NULL);
   g_assert (tubes_chan != NULL);
   g_object_ref (tubes_chan);
 
+  *created = TRUE;
   return tubes_chan;
 }
