@@ -70,6 +70,7 @@ static guint signals[LAST_SIGNAL] = {0};
 enum
 {
   PROP_CONNECTION = 1,
+  PROP_TUBES_CHANNEL,
   PROP_HANDLE,
   PROP_HANDLE_TYPE,
   PROP_SELF_HANDLE,
@@ -78,7 +79,7 @@ enum
   PROP_BYTESTREAM,
   PROP_STREAM_ID,
   PROP_TYPE,
-  PROP_INITIATOR,
+  PROP_INITIATOR_HANDLE,
   PROP_SERVICE,
   PROP_PARAMETERS,
   PROP_STATE,
@@ -92,6 +93,7 @@ typedef struct _SalutTubeDBusPrivate SalutTubeDBusPrivate;
 struct _SalutTubeDBusPrivate
 {
   SalutConnection *conn;
+  SalutTubesChannel *tubes_channel;
   TpHandle handle;
   TpHandleType handle_type;
   TpHandle self_handle;
@@ -130,7 +132,7 @@ struct _SalutTubeDBusPrivate
 };
 
 #define SALUT_TUBE_DBUS_GET_PRIVATE(obj) \
-    ((SalutTubeDBusPrivate *) ((SalutTubeDBus *)obj)->priv)
+    ((SalutTubeDBusPrivate *) ((SalutTubeDBus *) obj)->priv)
 
 static void data_received_cb (GibberBytestreamIface *bytestream,
     const gchar *from, GString *data, gpointer user_data);
@@ -485,8 +487,6 @@ salut_tube_dbus_dispose (GObject *object)
 
   if (priv->dbus_names)
     {
-      TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (
-          (TpBaseConnection *) priv->conn, TP_HANDLE_TYPE_CONTACT);
       g_hash_table_foreach (priv->dbus_names, unref_handle_foreach,
           contact_repo);
       g_hash_table_destroy (priv->dbus_names);
@@ -536,6 +536,9 @@ salut_tube_dbus_get_property (GObject *object,
       case PROP_CONNECTION:
         g_value_set_object (value, priv->conn);
         break;
+      case PROP_TUBES_CHANNEL:
+        g_value_set_object (value, priv->tubes_channel);
+        break;
       case PROP_HANDLE:
         g_value_set_uint (value, priv->handle);
         break;
@@ -560,7 +563,7 @@ salut_tube_dbus_get_property (GObject *object,
       case PROP_TYPE:
         g_value_set_uint (value, TP_TUBE_TYPE_DBUS);
         break;
-      case PROP_INITIATOR:
+      case PROP_INITIATOR_HANDLE:
         g_value_set_uint (value, priv->initiator);
         break;
       case PROP_SERVICE:
@@ -601,6 +604,9 @@ salut_tube_dbus_set_property (GObject *object,
       case PROP_CONNECTION:
         priv->conn = g_value_get_object (value);
         break;
+      case PROP_TUBES_CHANNEL:
+        priv->tubes_channel = g_value_get_object (value);
+        break;
       case PROP_HANDLE:
         priv->handle = g_value_get_uint (value);
         break;
@@ -635,7 +641,7 @@ salut_tube_dbus_set_property (GObject *object,
                 G_CALLBACK (bytestream_state_changed_cb), self);
           }
         break;
-      case PROP_INITIATOR:
+      case PROP_INITIATOR_HANDLE:
         priv->initiator = g_value_get_uint (value);
         break;
       case PROP_SERVICE:
@@ -643,7 +649,9 @@ salut_tube_dbus_set_property (GObject *object,
         priv->service = g_value_dup_string (value);
         break;
       case PROP_PARAMETERS:
-        priv->parameters = g_value_get_boxed (value);
+        if (priv->parameters != NULL)
+          g_hash_table_destroy (priv->parameters);
+        priv->parameters = g_value_dup_boxed (value);
         break;
       default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
@@ -760,6 +768,8 @@ salut_tube_dbus_class_init (SalutTubeDBusClass *salut_tube_dbus_class)
 
   g_object_class_override_property (object_class, PROP_CONNECTION,
     "connection");
+  g_object_class_override_property (object_class, PROP_TUBES_CHANNEL,
+    "tubes-channel");
   g_object_class_override_property (object_class, PROP_HANDLE,
     "handle");
   g_object_class_override_property (object_class, PROP_HANDLE_TYPE,
@@ -770,8 +780,8 @@ salut_tube_dbus_class_init (SalutTubeDBusClass *salut_tube_dbus_class)
     "id");
   g_object_class_override_property (object_class, PROP_TYPE,
     "type");
-  g_object_class_override_property (object_class, PROP_INITIATOR,
-    "initiator");
+  g_object_class_override_property (object_class, PROP_INITIATOR_HANDLE,
+    "initiator-handle");
   g_object_class_override_property (object_class, PROP_SERVICE,
     "service");
   g_object_class_override_property (object_class, PROP_PARAMETERS,
@@ -786,10 +796,7 @@ salut_tube_dbus_class_init (SalutTubeDBusClass *salut_tube_dbus_class)
       "tube if it has a HANDLE_TYPE_ROOM handle",
       GIBBER_TYPE_MUC_CONNECTION,
       G_PARAM_CONSTRUCT_ONLY |
-      G_PARAM_READWRITE |
-      G_PARAM_STATIC_NAME |
-      G_PARAM_STATIC_NICK |
-      G_PARAM_STATIC_BLURB);
+      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
   g_object_class_install_property (object_class, PROP_MUC_CONNECTION,
       param_spec);
 
@@ -798,10 +805,7 @@ salut_tube_dbus_class_init (SalutTubeDBusClass *salut_tube_dbus_class)
       "Object implementing the GibberBytestreamIface interface",
       "Bytestream object used for streaming data for this D-Bus",
       G_TYPE_OBJECT,
-      G_PARAM_READWRITE |
-      G_PARAM_STATIC_NAME |
-      G_PARAM_STATIC_NICK |
-      G_PARAM_STATIC_BLURB);
+      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
   g_object_class_install_property (object_class, PROP_BYTESTREAM, param_spec);
 
   param_spec = g_param_spec_string (
@@ -809,10 +813,7 @@ salut_tube_dbus_class_init (SalutTubeDBusClass *salut_tube_dbus_class)
       "stream id",
       "The identifier of this tube's bytestream",
       "",
-      G_PARAM_READABLE |
-      G_PARAM_STATIC_NAME |
-      G_PARAM_STATIC_NICK |
-      G_PARAM_STATIC_BLURB);
+      G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
   g_object_class_install_property (object_class, PROP_STREAM_ID, param_spec);
 
   param_spec = g_param_spec_string (
@@ -820,10 +821,7 @@ salut_tube_dbus_class_init (SalutTubeDBusClass *salut_tube_dbus_class)
       "D-Bus address",
       "The D-Bus address on which this tube will listen for connections",
       "",
-      G_PARAM_READABLE |
-      G_PARAM_STATIC_NAME |
-      G_PARAM_STATIC_NICK |
-      G_PARAM_STATIC_BLURB);
+      G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
   g_object_class_install_property (object_class, PROP_DBUS_ADDRESS,
       param_spec);
 
@@ -832,10 +830,7 @@ salut_tube_dbus_class_init (SalutTubeDBusClass *salut_tube_dbus_class)
       "D-Bus name",
       "The local D-Bus name on the virtual bus.",
       "",
-      G_PARAM_READABLE |
-      G_PARAM_STATIC_NAME |
-      G_PARAM_STATIC_NICK |
-      G_PARAM_STATIC_BLURB);
+      G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
   g_object_class_install_property (object_class, PROP_DBUS_NAME, param_spec);
 
   param_spec = g_param_spec_boxed (
@@ -843,14 +838,11 @@ salut_tube_dbus_class_init (SalutTubeDBusClass *salut_tube_dbus_class)
       "D-Bus names",
       "Mapping of contact handles to D-Bus names.",
       G_TYPE_HASH_TABLE,
-      G_PARAM_READABLE |
-      G_PARAM_STATIC_NAME |
-      G_PARAM_STATIC_NICK |
-      G_PARAM_STATIC_BLURB);
+      G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
   g_object_class_install_property (object_class, PROP_DBUS_NAMES, param_spec);
 
   signals[OPENED] =
-    g_signal_new ("opened",
+    g_signal_new ("tube-opened",
                   G_OBJECT_CLASS_TYPE (salut_tube_dbus_class),
                   G_SIGNAL_RUN_LAST | G_SIGNAL_DETAILED,
                   0,
@@ -859,7 +851,7 @@ salut_tube_dbus_class_init (SalutTubeDBusClass *salut_tube_dbus_class)
                   G_TYPE_NONE, 0);
 
   signals[CLOSED] =
-    g_signal_new ("closed",
+    g_signal_new ("tube-closed",
                   G_OBJECT_CLASS_TYPE (salut_tube_dbus_class),
                   G_SIGNAL_RUN_LAST | G_SIGNAL_DETAILED,
                   0,
@@ -1051,7 +1043,7 @@ data_received_cb (GibberBytestreamIface *stream,
             {
               DEBUG ("D-Bus message has unknown endianness byte 0x%x, "
                   "closing tube", (unsigned int) buf->str[0]);
-              salut_tube_iface_close (SALUT_TUBE_IFACE (tube));
+              salut_tube_iface_close (SALUT_TUBE_IFACE (tube), FALSE);
               return;
             }
 
@@ -1072,7 +1064,7 @@ data_received_cb (GibberBytestreamIface *stream,
               priv->reassembly_bytes_needed > DBUS_MAXIMUM_MESSAGE_LENGTH)
             {
               DEBUG ("D-Bus message is too large to be valid, closing tube");
-              salut_tube_iface_close (SALUT_TUBE_IFACE (tube));
+              salut_tube_iface_close (SALUT_TUBE_IFACE (tube), FALSE);
               return;
             }
 
@@ -1092,6 +1084,7 @@ data_received_cb (GibberBytestreamIface *stream,
 
 SalutTubeDBus *
 salut_tube_dbus_new (SalutConnection *conn,
+                     SalutTubesChannel *tubes_channel,
                      TpHandle handle,
                      TpHandleType handle_type,
                      TpHandle self_handle,
@@ -1099,24 +1092,21 @@ salut_tube_dbus_new (SalutConnection *conn,
                      TpHandle initiator,
                      const gchar *service,
                      GHashTable *parameters,
-                     guint id,
-                     GibberBytestreamIface *bytestream)
+                     guint id)
 {
   SalutTubeDBus *tube = g_object_new (SALUT_TYPE_TUBE_DBUS,
       "connection", conn,
+      "tubes-channel", tubes_channel,
       "handle", handle,
       "handle-type", handle_type,
       "self-handle", self_handle,
       "muc-connection", muc_connection,
-      "initiator", initiator,
+      "initiator-handle", initiator,
       "service", service,
       "parameters", parameters,
       "id", id,
       NULL);
 
-  if (bytestream != NULL)
-    g_object_set (tube, "bytestream", bytestream, NULL);
-
   return tube;
 }
 
@@ -1164,7 +1154,7 @@ salut_tube_dbus_accept (SalutTubeIface *tube,
  * Implements salut_tube_iface_close on SalutTubeIface
  */
 static void
-salut_tube_dbus_close (SalutTubeIface *tube)
+salut_tube_dbus_close (SalutTubeIface *tube, gboolean closed_remotely)
 {
   SalutTubeDBus *self = SALUT_TUBE_DBUS (tube);
   SalutTubeDBusPrivate *priv = SALUT_TUBE_DBUS_GET_PRIVATE (self);
@@ -1276,6 +1266,7 @@ tube_iface_init (gpointer g_iface,
   SalutTubeIfaceClass *klass = (SalutTubeIfaceClass *) g_iface;
 
   klass->accept = salut_tube_dbus_accept;
+  klass->offer_needed = NULL;
   klass->close = salut_tube_dbus_close;
   klass->add_bytestream = salut_tube_dbus_add_bytestream;
 }
