@@ -92,6 +92,12 @@ static const gchar *salut_tube_stream_interfaces[] = {
     NULL
 };
 
+static const gchar * const salut_tube_stream_channel_allowed_properties[] = {
+    TP_IFACE_CHANNEL ".TargetHandle",
+    TP_IFACE_CHANNEL ".TargetID",
+    SALUT_IFACE_CHANNEL_TYPE_STREAM_TUBE ".Service",
+    NULL
+};
 
 /* Linux glibc bits/socket.h suggests that struct sockaddr_storage is
  * not guaranteed to be big enough for AF_UNIX addresses */
@@ -109,6 +115,7 @@ enum
   OPENED,
   NEW_CONNECTION,
   CLOSED,
+  OFFERED,
   LAST_SIGNAL
 };
 
@@ -614,7 +621,7 @@ start_stream_direct (SalutTubeStream *self,
   g_assert (bytestream != NULL);
 
   g_hash_table_insert (priv->bytestream_to_transport,
-      g_object_ref (bytestream), g_object_ref (transport));
+      bytestream, g_object_ref (transport));
 
   g_signal_connect (bytestream, "state-changed",
       G_CALLBACK (extra_bytestream_state_changed_cb), self);
@@ -974,6 +981,7 @@ salut_tube_stream_finalize (GObject *object)
   SalutTubeStream *self = SALUT_TUBE_STREAM (object);
   SalutTubeStreamPrivate *priv = SALUT_TUBE_STREAM_GET_PRIVATE (self);
 
+  g_free (priv->object_path);
   g_free (priv->service);
   if (priv->parameters != NULL)
     {
@@ -1353,28 +1361,12 @@ salut_tube_stream_constructor (GType type,
       ((TpBaseConnection *) priv->conn, TP_HANDLE_TYPE_CONTACT);
   tp_handle_ref (contact_repo, priv->initiator);
 
-  /* Set initial state of the tube */
   if (priv->initiator == priv->self_handle)
     {
       /* We initiated this tube */
-      if (priv->handle_type == TP_HANDLE_TYPE_CONTACT)
-        {
-          /* Private tube */
-          if (priv->offered)
-            {
-              priv->state = SALUT_TUBE_CHANNEL_STATE_REMOTE_PENDING;
-              priv->offer_needed = TRUE;
-            }
-          else
-            {
-              priv->state = SALUT_TUBE_CHANNEL_STATE_NOT_OFFERED;
-            }
-        }
-      else
-        {
-          /* Muc tube */
-          priv->state = SALUT_TUBE_CHANNEL_STATE_OPEN;
-        }
+      priv->state = SALUT_TUBE_CHANNEL_STATE_NOT_OFFERED;
+      /* FIXME: we should probably remove this offer_needed */
+      priv->offer_needed = TRUE;
     }
   else
     {
@@ -1638,6 +1630,15 @@ salut_tube_stream_class_init (SalutTubeStreamClass *salut_tube_stream_class)
                   salut_signals_marshal_VOID__VOID,
                   G_TYPE_NONE, 0);
 
+  signals[OFFERED] =
+    g_signal_new ("tube-offered",
+                  G_OBJECT_CLASS_TYPE (salut_tube_stream_class),
+                  G_SIGNAL_RUN_LAST | G_SIGNAL_DETAILED,
+                  0,
+                  NULL, NULL,
+                  salut_signals_marshal_VOID__VOID,
+                  G_TYPE_NONE, 0);
+
   salut_tube_stream_class->dbus_props_class.interfaces = prop_interfaces;
   tp_dbus_properties_mixin_class_init (object_class,
       G_STRUCT_OFFSET (SalutTubeStreamClass, dbus_props_class));
@@ -1801,6 +1802,10 @@ salut_tube_stream_accept (SalutTubeIface *tube,
 
   priv->state = SALUT_TUBE_CHANNEL_STATE_OPEN;
   g_signal_emit (G_OBJECT (self), signals[OPENED], 0);
+
+  salut_svc_channel_interface_tube_emit_tube_channel_state_changed (
+      self, SALUT_TUBE_CHANNEL_STATE_OPEN);
+
   return TRUE;
 }
 
@@ -1820,6 +1825,9 @@ salut_tube_stream_accepted (SalutTubeIface *tube)
 
   priv->state = TP_TUBE_STATE_OPEN;
   g_signal_emit (G_OBJECT (self), signals[OPENED], 0);
+
+  salut_svc_channel_interface_tube_emit_tube_channel_state_changed (
+      self, SALUT_TUBE_CHANNEL_STATE_OPEN);
 }
 
 /**
@@ -1879,6 +1887,7 @@ contact_new_connection_cb (GibberListener *listener,
   salut_tube_stream_add_bytestream (SALUT_TUBE_IFACE (self), bytestream);
   gibber_bytestream_direct_accept_socket (bytestream, transport);
 
+  g_object_unref (bytestream);
   g_object_unref (contact);
   g_object_unref (contact_mgr);
 }
@@ -2043,6 +2052,9 @@ salut_tube_stream_add_bytestream (SalutTubeIface *tube,
 
       g_signal_emit (G_OBJECT (self), signals[NEW_CONNECTION], 0, contact);
 
+      salut_svc_channel_type_stream_tube_emit_stream_tube_new_connection (
+          self, contact);
+
       tp_handle_unref (contact_repo, contact);
       g_free (peer_id);
     }
@@ -2229,15 +2241,6 @@ salut_tube_stream_check_params (TpSocketAddressType address_type,
     }
 }
 
-static void
-stream_tube_new_connection_cb (SalutTubesChannel *self,
-                               guint contact,
-                               gpointer user_data)
-{
-  salut_svc_channel_type_stream_tube_emit_stream_tube_new_connection (
-      self, contact);
-}
-
 /**
  * salut_tube_stream_offer_stream_tube
  *
@@ -2274,11 +2277,6 @@ salut_tube_stream_offer_stream_tube (SalutSvcChannelTypeStreamTube *iface,
       return;
     }
 
-  if (priv->handle_type == TP_HANDLE_TYPE_ROOM)
-    priv->state = SALUT_TUBE_CHANNEL_STATE_OPEN;
-  else
-    priv->state = SALUT_TUBE_CHANNEL_STATE_REMOTE_PENDING;
-
   g_assert (address_type == TP_SOCKET_ADDRESS_TYPE_UNIX ||
       address_type == TP_SOCKET_ADDRESS_TYPE_IPV4 ||
       address_type == TP_SOCKET_ADDRESS_TYPE_IPV6);
@@ -2292,17 +2290,13 @@ salut_tube_stream_offer_stream_tube (SalutSvcChannelTypeStreamTube *iface,
 
   g_object_set (self, "parameters", parameters, NULL);
 
-  if (priv->handle_type == TP_HANDLE_TYPE_CONTACT)
+  if (!salut_tube_stream_offer (self, &error))
     {
-      salut_tubes_channel_send_iq_offer (priv->tubes_channel);
-
-      salut_svc_channel_interface_tube_emit_tube_channel_state_changed (
-          self, SALUT_TUBE_CHANNEL_STATE_REMOTE_PENDING);
+      dbus_g_method_return_error (context, error);
+      g_error_free (error);
+      return;
     }
 
-  g_signal_connect (self, "new-connection",
-      G_CALLBACK (stream_tube_new_connection_cb), self);
-
   salut_svc_channel_type_stream_tube_return_from_offer_stream_tube (context);
 }
 
@@ -2353,6 +2347,12 @@ salut_tube_stream_accept_stream_tube (SalutSvcChannelTypeStreamTube *iface,
       return;
     }
 
+  g_object_set (self,
+      "address-type", address_type,
+      "access-control", access_control,
+      "access-control-param", access_control_param,
+      NULL);
+
   if (!salut_tube_stream_accept (SALUT_TUBE_IFACE (self), &error))
     {
       dbus_g_method_return_error (context, error);
@@ -2485,6 +2485,41 @@ salut_tube_stream_get_supported_socket_types (void)
   return ret;
 }
 
+gboolean
+salut_tube_stream_offer (SalutTubeStream *self,
+                         GError **error)
+{
+  SalutTubeStreamPrivate *priv = SALUT_TUBE_STREAM_GET_PRIVATE (self);
+
+  g_assert (priv->state == SALUT_TUBE_CHANNEL_STATE_NOT_OFFERED);
+
+  if (priv->handle_type == TP_HANDLE_TYPE_CONTACT)
+    {
+      priv->state = SALUT_TUBE_CHANNEL_STATE_REMOTE_PENDING;
+      salut_tubes_channel_send_iq_offer (priv->tubes_channel);
+
+      salut_svc_channel_interface_tube_emit_tube_channel_state_changed (
+          self, SALUT_TUBE_CHANNEL_STATE_REMOTE_PENDING);
+    }
+  else
+    {
+      /* muc tube is open as soon it's offered */
+      priv->state = SALUT_TUBE_CHANNEL_STATE_OPEN;
+      salut_svc_channel_interface_tube_emit_tube_channel_state_changed (
+          self, SALUT_TUBE_CHANNEL_STATE_OPEN);
+      g_signal_emit (G_OBJECT (self), signals[OPENED], 0);
+    }
+
+  g_signal_emit (G_OBJECT (self), signals[OFFERED], 0);
+  return TRUE;
+}
+
+const gchar * const *
+salut_tube_stream_channel_get_allowed_properties (void)
+{
+  return salut_tube_stream_channel_allowed_properties;
+}
+
 static void
 tube_iface_init (gpointer g_iface,
                  gpointer iface_data)
