@@ -176,7 +176,10 @@ class ReceiveFileTest(FileTransferTest):
                 self_.end_headers()
                 self_.wfile.write(self.file.data)
 
-        self.httpd = BaseHTTPServer.HTTPServer(('', 0), HTTPHandler)
+        self.httpd = self._get_http_server_class()(('', 0), HTTPHandler)
+
+    def _get_http_server_class(self):
+        return BaseHTTPServer.HTTPServer
 
     def send_ft_offer_iq(self):
         iq = domish.Element((None, 'iq'))
@@ -231,7 +234,7 @@ class ReceiveFileTest(FileTransferTest):
 
     def accept_file(self):
         self.address = self.ft_channel.AcceptFile(SOCKET_ADDRESS_TYPE_UNIX,
-                SOCKET_ACCESS_CONTROL_LOCALHOST, "", 5)
+                SOCKET_ACCESS_CONTROL_LOCALHOST, "", 5, byte_arrays=True)
 
         e = self.q.expect('dbus-signal', signal='FileTransferStateChanged')
         state, reason = e.args
@@ -380,7 +383,7 @@ class SendFileTest(FileTransferTest):
 
     def provide_file(self):
         self.address = self.ft_channel.ProvideFile(SOCKET_ADDRESS_TYPE_UNIX,
-                SOCKET_ACCESS_CONTROL_LOCALHOST, "")
+                SOCKET_ACCESS_CONTROL_LOCALHOST, "", byte_arrays=True)
 
     def client_request_file(self):
         # Connect HTTP client to the CM and request the file
